/*A Gate is any sort of a logic gate. A few more functions are defined.
 * The Gate constructor defines a gate with two inputs and a single output.
 */
public abstract class Gate extends Object {
  //Delay is usually -1. When inputs change, delay counts 
  //down from propTime to 0, then outputs change.
  int delay = 0;

  /**The constructor must declare how many inputs there are, and 
   	 * manages whatever else comes up.
   	 */
  public Gate(int x, int y) {
    super(x, y);
    output = new Output[1];		
    output[0] = new Output(this, .5, 0);

    input = new Input[2];
    input[0] = new Input(this, -.5, -.25);
    input[1] = new Input(this, -.5, .25);
  }

  //Assert outputs one propagation delay after inputs change. 
  public void tick() {
    for (int i = 0; i < output.length; i++) {
      //Emulate propagation delay
      if ((findState(i) != output[i].get()) && (delay == -1)) delay = propTime();

      //Switch outputs after the timer runs up.
      if (delay == 0) {
        output[i].set(findState(i));
      }
    }
    if (delay >= 0) delay--;
  }

  /**Returns the propagation delay time of a gate.*/
  protected int propTime() {
    return 10;
  }

  protected boolean findState() {
    return findState(0);
  }

  /**Returns the nth output value, 
   	 * based only on inputs (ignoring delays & clock shenanigans)
   	 */
  protected abstract boolean findState(int index);

  protected void setPen() {
    super.setPen();
    stroke(GATE);
  }
}

