/**Input hotspots provide inputs to gates. They associate with only a single
 * Output Hotspot, and cannot be set directly.*/
public class Input extends Hotspot {	
  Output partner;

  public Input(Object obj, float dx, float dy) {
    super(obj, dx, dy);
    name = "Input";
    DISCONNECTED = color(255, 125, 226);
    inputs.add(this);
  }

  public void tick() {
  };

  /**Adds a line between inputs and outputs to the circles drawn in the superclass.*/
  public void draw() {
    setPen();
    //Draw lines.

    if (attached() && partner.get()) stroke(TRUE);
    else stroke(FALSE);
    if (partner != null) line(getX(), getY(), partner.getX(), partner.getY());
    //Draw bubbles.
    super.draw();
  }

  public void associate() {		
    //If this input's output moves out of range, dissociate. 
    if (attached() && (dist(partner.getX(), partner.getY(), 
    getX(), getY()) > HOTSPOT_GRAB_RADIUS)) {
      detach();
    }

    //If dissociated, search for a new partner.
    if (!attached()) {
      for (Output output : outputs) {
        if (dist(output.getX(), output.getY(), 
        getX(), getY()) < HOTSPOT_GRAB_RADIUS) {
          attach(output);
          break;
        }
      }
    }
  }

  public void attach(Hotspot hot) {
    if (hot.getParent() == getParent()) return;
    if (hot instanceof Output) partner = (Output) hot;
    if (!hot.attached(this)) hot.attach(this);
  }

  public void detach(Hotspot hot) {
    if (hot == partner) partner = null;
    if (hot.attached(this)) hot.detach(this);
  }

  public void detach() {
    if (partner != null && partner.attached(this)) partner.detach(this);
    partner = null;
  }

  public boolean attached(Hotspot hot) {
    return (partner == hot);
  }

  public boolean attached() {
    return (partner != null);
  }

  public void remove() {
    things.remove(this);
    inputs.remove(this);
    detach();
  }

  public boolean get() {
    //Not technically correct: if a gate is left floating, the input is indeterminate.
    //I'm emulating a pull-down resistor to make simulation work.
    boolean val;
    if (partner == null) val = false;
    else val = partner.get();
    if (invert)	return !val;
    else		return  val;
  }


  public String toString() {
    if (partner == null) return super.toString() + "\n" + " Attached to: null";
    return super.toString() + "\n" + " Attached to: " + partner.shortString();
  }
}

