import java.util.*;

//Main code of LogicSim II
/*
Changelog
 1: Initial release
 2: Fixed wire association problems
    Added colors to jumper wires
    Limited sensitivity of UserInputs to just over the input
    Added inversion ability to Hotspots.
 3: Added grid snapping
    Added OSD & helptext
 */
//Whether a Object is selected.
boolean 
  dirty = true, 
  drawGrid = false;


PGraphics gridCanvas;
PFont font;
OSD help;

Object active = null; //The currently selected object. 

//Inputs and outputs. Used to figure out associations.
List<Input>  inputs   = new LinkedList<Input>();
List<Output> outputs  = new LinkedList<Output>();
List<OSD>    displays = new LinkedList<OSD>();

//All the things on the canvas, except for inputs and outputs. 
List<Object> things = new LinkedList<Object>();
List<Object> kill   = new LinkedList<Object>();

//Universal constants
final int 
  MOUSE_GRAB_RADIUS   = 30, 
  HOTSPOT_GRAB_RADIUS = 15,
//Grid drawing modes.
  OFF  = 2, //No snapping
  SNAP = 1, //Snapping; invisible grid
  DRAW = 0; //Snapping; visible grid.

final float
  HOTSPOT_RADIUS_PER_GATE_SIZE = .1,
  GRID_SPACING = 30/4;

final color
  TRUE  = color(0, 255, 0), 
  FALSE = color(255, 0, 0), 
  GATE  = color(  0, 0, 0), 
  ATTN  = color(0, 0, 100),
  TEXT  = color(0, 0,   0);

//The clock ticks every n frames. 
int 
  clockPeriod = 1,
  gridMode = 0;

void setup() {
  size(1000, 600);
  background(255);
  frameRate(60);
  
  strokeCap(ROUND);
  grid();
  
  font = loadFont("font.vlw");
  textFont(font);
  
  help = new OSD(10, height-30, "Placement: (w)ire, (n)ot, o(r), (x)or, (a)nd, (i)nput, (o)utput\nOther:     S/s - increase/decrease simulation speed   g - snap gates / draw grid / snap off  h - hide this text");
}

void draw() {
  //Handle asynchronous redrawing. 
  if (dirty) sketch();

  //1ns elapses every clockPeriod frames.
  if (frameCount % clockPeriod == 0) {
    for (Object o : things) o.tick();
  }
  //Tick OSDs every single frame.
  for (OSD o : displays) o.tick();
  
  //Cleanup.
  for (Object o : kill) o.remove();
  kill.clear();
}

void sketch() {
  background(255);
  if (gridMode == DRAW) image(gridCanvas, 0, 0); //Draw grid spacing.

  for (Object thing : things) thing.draw();
  dirty = false;
}

/**Draws a grid onto the screen.*/
void grid() {
  gridCanvas = createGraphics(width, height);
  gridCanvas.beginDraw(); 
  float x=0, y=0;
  while (x < width) {
    y = 0;
    while (y < height) {
      gridCanvas.point(x, y);
      y+=GRID_SPACING;
    }
    x+=GRID_SPACING;
  }
  gridCanvas.endDraw();
}

//Pass mouse events on to objects. 
//Old-style loops allow for modification during iteration (for deleting stuff)
void mousePressed() {
  for (int i = 0; i < things.size(); i++) things.get(i).mousePressed();
}
void mouseReleased() {
  for (int i = 0; i < things.size(); i++) things.get(i).mouseReleased();
}
void mouseMoved() {
  for (int i = 0; i < things.size(); i++) things.get(i).mouseMoved();
}
void mouseDragged() {
  for (int i = 0; i < things.size(); i++) things.get(i).mouseDragged();
}

//Handles logic for placing things on the board. 
void keyPressed() {
  //Create a new gate. 
  if (key == 'w') new Wire(mouseX, mouseY);
  if (key == 'n') new Not(mouseX, mouseY);
  if (key == 'r') new Or(mouseX, mouseY);
  if (key == 'x') new XOr(mouseX, mouseY);
  if (key == 'a') new And(mouseX, mouseY);
  if (key == 'i') new UserInput(mouseX, mouseY);
  if (key == 'o') new UserOutput(mouseX, mouseY);
  if (key == 't') new OSD(mouseX, mouseY, "Hello!", 3, 1);
  if (key == 's') clockPeriod++;
  if ((clockPeriod > 1) && (key == 'S')) clockPeriod--;
  if (key == 's' || key == 'S') new OSD(3, 10, "Current clock period: "+clockPeriod+" frames per tick", 3, .5);
  
  
  if (key == 'g') {
    gridMode = (gridMode + 1) %3;
    String message = "";
    if (gridMode ==  OFF) message = "Constraints disabled\nGrid disabled";
    if (gridMode == DRAW) message = "Constraints enabled \nGrid enabled ";
    if (gridMode == SNAP) message = "Constraints enabled \nGrid disabled";
    new OSD(3, 10, message, 3, .5);
  
    sketch();
  }
  
  //Remove help
  if (key == 'h') help.fadeOut(1);
}

