/**An OSD provides a method to display text on the screen. OSD's are minimally interactive, and will vanish on their own after timeout expires.*/

public class OSD extends Object {
  int fadeStart, fadeEnd;
  /**Constructor for an Onscreen display
  @param x The x-coordinate of the text
  @param y The y-coordinate of the text
  @param message The message to display
  @param timeout The amount of time to display the message for
  @param fade The time to fade for, before vanishing. 
  */
  public OSD(int x, int y, String message, float timeout, float fade) {
    super(x, y);
    name = message;
    
    if (timeout == -1) {
      fadeEnd = -1;
      fadeStart = -1;
    } else {
      fadeEnd   = (int) (frameCount + frameRate * timeout);
      fadeStart = (int) (fadeEnd -  frameRate * fade);      
    }
  }
  
  /**Constructor for an Onscreen display
  @param x The x-coordinate of the text
  @param y The y-coordinate of the text
  @param message The message to display
  @param timeout The amount of time to display the message for
  */
  public OSD(int x, int y, String message, float timeout) {
    this(x, y, message, timeout, 0);
  }  

  /**Constructor for an Onscreen display
  @param x The x-coordinate of the text
  @param y The y-coordinate of the text
  @param message The message to display
  @param timeout The amount of time to display the message for
  */
  public OSD(int x, int y, String message) {
    this(x, y, message, -1, 0);
  }  
  
  public void tick() {
    //Flag this for removal. We'll periodically poll for removal.
    if (fadeEnd != -1 && frameCount > fadeEnd) kill.add(this);
    else if (getAlpha() < 255) dirty = true;  
  }
  
  public int getAlpha() {
    if (fadeEnd == -1) return 255; //Support for timeless notices.
    else return (int) (255 * ((fadeEnd - frameCount) / ((float) (fadeEnd - fadeStart))));
  }
  
  public void setPen() {
    super.setPen();
    fill(red(TEXT), green(TEXT), blue(TEXT), getAlpha());
  }
  
  private void setPen2() {
    fill(255, 255, 255, getAlpha());
    noStroke();
  }
  
  public void draw() {
    setPen2();
    rect(getX()-3, getY()-10, textWidth(name)+6, name.split("\n").length*14+3);
    
    setPen();
    text(name, getX(), getY());
  }  
  
  public void remove() {
    things.remove(this);
    displays.remove(this);
    dirty = true;
  }
  
  /**Requests for an OSD to fade out
  @param fade The number of seconds to fade for before vanishing.
  */
  public void fadeOut(float fade) {
    fadeEnd   = (int) (frameCount + frameRate * fade);
    fadeStart = frameCount;
  }
}
