/**Input hotspots provide inputs to gates. They associate with only a single
 * Output Hotspot, and cannot be set directly.*/
public class Output extends Hotspot {	
  List<Input> partners = new ArrayList<Input>();
  boolean state = false;

  public Output(Object obj, float dx, float dy) {
    super(obj, dx, dy);
    name = "Output";
    DISCONNECTED = color(125, 255, 134);
    outputs.add(this);
  }

  public void tick() {
  }

  public void associate() {
    //Remove out-of-range elements
    for (int i=0; i < partners.size(); i++) {
      if (dist(partners.get(i).getX(), partners.get(i).getY(), 
      getX(), getY()) > HOTSPOT_GRAB_RADIUS) {
        detach(partners.get(i));
        i--; //This index now refers to a different element. Check it.
      }
    }

    //Add in-range elements.
    for (Input input : inputs) {
      if (!input.attached() 
        &&(dist(input.getX(), input.getY(), getX(), getY()) < HOTSPOT_GRAB_RADIUS)
        && !partners.contains(input)) {
        attach(input);
      }
    }
  }

  public void attach(Hotspot hot) {
    if (hot.getParent() == getParent()) return;
    if (hot instanceof Input) {
      partners.add((Input) hot);
      if (!hot.attached(this)) hot.attach(this);
    }
  }

  public void detach(Hotspot hot) {
    partners.remove(hot);
    if (hot.attached(this)) hot.detach(this);
  }

  public void remove() {
    things.remove(this);
    outputs.remove(this);
    while (partners.size () != 0) detach(partners.get(0));
  }

  public boolean attached(Hotspot hot) {
    return partners.contains(hot);
  }

  public boolean attached() {
    return !partners.isEmpty();
  }
  public boolean get() {
    if (invert) return !state;
    else		return	state;
  }

  /**Sets the state of this Output to newState. The change instantly 
   	 * propogates to all associated Inputs. 
   	 * @param newState The new state of the net. 
   	 */
  public void set(boolean newState) {
    state = newState;
    dirty = true;
  }

  public String toString() {
    String s = super.toString();
    for (Input i : inputs) s+= "\n Attached to: " + i.shortString();
    return s;
  }
}

